# This file only contains a selection of the most common options. For a full
# list see the documentation:
#
# https://www.sphinx-doc.org/en/master/usage/configuration.html


# -- Path setup --------------------------------------------------------------

import os
import sys

# Add parent directory so we can import bleach
sys.path.insert(0, os.path.abspath('..'))

import bleach  # noqa

# -- Project information -----------------------------------------------------

project = "Bleach"
copyright = "2012-2015, James Socol; 2015-2017, Mozilla Foundation"
author = "Mozilla Foundation"


# -- General configuration -----------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = ['sphinx.ext.autodoc', 'sphinx.ext.viewcode', 'sphinx.ext.doctest']

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = ['_build']

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
# The short X.Y version.
version = bleach.__version__
# The full version, including alpha/beta/rc tags.
release = bleach.__version__ + ' ' + bleach.__releasedate__

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# Display the class docstring and __init__ docstring concatenated
autoclass_content = 'both'
# Reduce complexity of function signatures by not evaluating the argument
# default values.
# NOTE(willkg) This is experimental in Sphinx 4.
autodoc_preserve_defaults = True


# -- Options for HTML output ---------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
html_theme = 'alabaster'

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']

# Custom sidebar templates, maps document names to template names.
html_sidebars = {
    '**': [
        'about.html',
        'navigation.html',
        'relations.html',
        'searchbox.html',
    ]
}



from dataclasses import dataclass
import inspect

import pytest

from bleach.parse_shim import urlparse


@dataclass
class ParseResult:
    scheme: str = ""
    netloc: str = ""
    path: str = ""
    params: str = ""
    query: str = ""
    fragment: str = ""


# Tests from
# https://github.com/web-platform-tests/wpt/blob/master/url/resources/urltestdata.json
# commit ee566de4c5c65d7e8af8b2500f9b85a646ffeaa5


@pytest.mark.parametrize(
    "uri, expected",
    [
        ("", ParseResult()),
        ("http://example\t.\norg", ParseResult(scheme="http", netloc="example.org")),
        (
            "http://user:pass@foo:21/bar;par?b#c",
            ParseResult(
                scheme="http",
                netloc="user:pass@foo:21",
                path="/bar",
                params="par",
                query="b",
                fragment="c",
            ),
        ),
        ("https://test:@test", ParseResult(scheme="https", netloc="test:@test")),
        ("https://:@test", ParseResult(scheme="https", netloc=":@test")),
        (
            "non-special://test:@test/x",
            ParseResult(scheme="non-special", netloc="test:@test", path="/x"),
        ),
        (
            "non-special://:@test/x",
            ParseResult(scheme="non-special", netloc=":@test", path="/x"),
        ),
        ("http:foo.com", ParseResult(scheme="http", path="foo.com")),
        # NOTE(willkg): The wpt tests set the scheme to http becaue that's what
        # the base url is. Since our parser is not using a baseurl, it sets the
        # scheme to "". Further, our parser includes spaces at the beginning,
        # but I don't see that as being problematic.
        ("\t   :foo.com   \n", ParseResult(path="   :foo.com   ")),
        # NOTE(willkg): The wpt tests set the path to "/foo/foo.com" because
        # the base url is at "/foo"
        (" foo.com  ", ParseResult(path=" foo.com  ")),
        ("a:\t foo.com", ParseResult(scheme="a", path=" foo.com")),
        (
            "http://f:21/ b ? d # e ",
            ParseResult(
                scheme="http", netloc="f:21", path="/ b ", query=" d ", fragment=" e "
            ),
        ),
        (
            "lolscheme:x x#x x",
            ParseResult(scheme="lolscheme", path="x x", fragment="x x"),
        ),
        ("http://f:/c", ParseResult(scheme="http", netloc="f:", path="/c")),
        ("http://f:0/c", ParseResult(scheme="http", netloc="f:0", path="/c")),
        # NOTE(willkg): The wpt tests normalize the 0000000000000 to 0 so the
        # netloc should be "f:0".
        (
            "http://f:00000000000000/c",
            ParseResult(scheme="http", netloc="f:00000000000000", path="/c"),
        ),
        # NOTE(willkg): The wpt tests drop the 0000000000000000000 altogether
        # so the netloc should be "f".
        (
            "http://f:00000000000000000000080/c",
            ParseResult(scheme="http", netloc="f:00000000000000000000080", path="/c"),
        ),
        # This is an invalid ipv6 url
        ("http://2001::1]", ValueError),
        # NOTE(willkg): The wpt tests show this as a parse error, but our
        # parser "parses" it.
        ("http://f:b/c", ParseResult(scheme="http", netloc="f:b", path="/c")),
        # NOTE(willkg): The wpt tests show this as a parse error, but our
        # parser "parses" it.
        ("http://f: /c", ParseResult(scheme="http", netloc="f: ", path="/c")),
        # NOTE(willkg): The wpt tests show this as a parse error, but our
        # parser "parses" it.
        ("http://f:999999/c", ParseResult(scheme="http", netloc="f:999999", path="/c")),
    ],
)
def test_urlparse(uri, expected):
    if inspect.isclass(expected) and issubclass(expected, BaseException):
        with pytest.raises(expected):
            urlparse(uri)

    else:
        parsed = urlparse(uri)
        print(parsed)
        assert parsed.scheme == expected.scheme
        assert parsed.netloc == expected.netloc
        assert parsed.path == expected.path
        assert parsed.params == expected.params
        assert parsed.query == expected.query
        assert parsed.fragment == expected.fragment



import pytest

from bleach import clean, linkify


def test_japanese_safe_simple():
    assert clean("ヘルプとチュートリアル") == "ヘルプとチュートリアル"
    assert linkify("ヘルプとチュートリアル") == "ヘルプとチュートリアル"


def test_japanese_strip():
    assert clean("<em>ヘルプとチュートリアル</em>") == "<em>ヘルプとチュートリアル</em>"
    assert (
        clean("<span>ヘルプとチュートリアル</span>")
        == "&lt;span&gt;ヘルプとチュートリアル&lt;/span&gt;"
    )


def test_russian_simple():
    assert clean("Домашняя") == "Домашняя"
    assert linkify("Домашняя") == "Домашняя"


def test_mixed():
    assert clean("Домашняяヘルプとチュートリアル") == "Домашняяヘルプとチュートリアル"


def test_mixed_linkify():
    assert (
        linkify("Домашняя http://example.com ヘルプとチュートリアル")
        == 'Домашняя <a href="http://example.com" rel="nofollow">http://example.com</a> ヘルプとチュートリアル'
    )


@pytest.mark.parametrize(
    "uri",
    [
        "http://éxámplé.com/",
        "http://éxámplé.com/íàñá/",
        "http://éxámplé.com/íàñá/?foo=bar",
        "http://éxámplé.com/íàñá/?fóo=bár",
    ],
)
def test_url_utf8(uri):
    """Allow UTF8 characters in URLs themselves."""
    assert linkify(uri) == f'<a href="{uri}" rel="nofollow">{uri}</a>'



#!/usr/bin/env python

import os
import re

from setuptools import setup, find_packages


def get_long_desc():
    with open("README.rst", encoding="utf-8") as fp:
        desc = fp.read()
    desc += "\n\n"
    with open("CHANGES", encoding="utf-8") as fp:
        desc += fp.read()
    return desc


def get_version():
    fn = os.path.join("bleach", "__init__.py")
    vsre = r"""^__version__ = ['"]([^'"]*)['"]"""
    with open(fn, encoding="utf-8") as fp:
        version_file = fp.read()
    return re.search(vsre, version_file, re.M).group(1)


INSTALL_REQUIRES = [
    # html5lib requirements
    "webencodings",
]


EXTRAS_REQUIRE = {
    "css": [
        "tinycss2>=1.1.0,<1.5",
    ],
}


setup(
    name="bleach",
    version=get_version(),
    description="An easy safelist-based HTML-sanitizing tool.",
    long_description=get_long_desc(),
    long_description_content_type="text/x-rst",
    maintainer="Will Kahn-Greene",
    maintainer_email="willkg@mozilla.com",
    url="https://github.com/mozilla/bleach",
    license="Apache Software License",
    license_files=["LICENSE", "bleach/_vendor/html5lib-1.1.dist-info/LICENSE"],
    packages=find_packages(),
    include_package_data=True,
    package_data={"": ["README.rst"]},
    zip_safe=False,
    python_requires=">=3.10",
    install_requires=INSTALL_REQUIRES,
    extras_require=EXTRAS_REQUIRE,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: 3.13",
        "Programming Language :: Python :: 3.14",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)
